#include	"dmonitor.h"
#include	"dvap.h"
#include	"dvmega.h"

void	rig_close(void);
int	inet_send_buff_set(void);
void	inet_send_position_update (char call[]);
void	icom_init (void);
int	rig_open (void);
void	sind_rig_alive (void);
void	icom_reinit (void);
void    rig_write (int length, unsigned char buff[]);
void    rig_led_onoff(void);
void	inet_led_onoff(void);
int	getFifo (char str[]);
void	jitter_buff_init (void);
void	dvap_init(void);
void	dvap (void);
void	dvmega (void);
void	node (void);
int	rig_read (void);
void    dvap_write (int fd, unsigned char buff[], int length);
void    dvmega_write (int fd, unsigned char buff[], int length);
void	node_write (unsigned char buff[], int length);
void	dstar_data (unsigned char string[], int inet);
void    dstar_alt_header (unsigned char string[], int inet);
void	short_msg_wrt (void);
void	rig_not_connect_msg (void);
int	dvmega_init (void);
int	dvmega_init_recv (void);	
int	no_cmd_check (unsigned char cmd[]);
unsigned short int      crc_calc (char str[], int length);

char	init_pkt[] = {0xff, 0xff, 0xff};
char	alive[] = {0x02, 0x02, 0xff};

time_t  rig_alive_send_time;
int	rig_error_cnt;

int	icom_init_recv (void)
{
	int length;
	int	i;
	int	k;

	if (FD_ISSET (rig_fd, &read_set))
	{
		length = read (rig_fd, &rig_buff[rig_buff_pnt], 
			sizeof(rig_buff) - rig_buff_pnt);
		if (length == -1)
		{
			syslog(LOG_ERR, "rig read error %s", strerror(errno));
		} 
		if (length > 0)
		{
			rig_buff_pnt += length;
		}
		if (rig_buff_pnt >= 3)
		{
			for (i = 0 ; i < rig_buff_pnt - 2 ; i++)
			{
				if ((rig_buff[i] == 0xff)
					&& (rig_buff[i+1] == 0xff)
					&& (rig_buff[i+2] == 0xff))
				{
					k = rig_buff_pnt - i - 3;
					if (k > 0) 
					{
						memmove (&rig_buff[0], &rig_buff[i+3], k);
						rig_buff_pnt = k;
					}
					else
						rig_buff_pnt = 0;
					time (&rig_alive_recv);
					return TRUE;
				}
			}
                	memmove (&rig_buff[0], &rig_buff[rig_buff_pnt - 2], 2);
                	rig_buff_pnt  = 2;
			
		}
		if (length < 0)
		{
			if (errno == EAGAIN) return TRUE;
		}
	}
	return FALSE;
}
	
enum
{
	RIG_CHECK = 0,
	RIG_OPEN,
	ICOM_INIT,
	ICOM_INIT_DONE,
	DVAP_INIT,
	DVAP_INIT_DONE,
	DVMEGA_INIT,
	DVMEGA_INIT_DONE,
	NODE_INIT,
	NODE_INIT_DONE,
	HOLE_PUNCH_WAIT,
	RIG_READ
} rig_state = RIG_CHECK;


void	rig(void)
{
	time_t	cur_time;
	char	dummy[42];
	int	length;
	int	n;
	int	ret;
	struct	stat	st;

	switch (rig_state)
	{
		case RIG_CHECK:
			rig_state = RIG_OPEN;
			break;

		case RIG_OPEN:
			if (rig_open())
			{
				if (rig_type == ICOM) rig_state = ICOM_INIT;
				else if (rig_type == DVAP) rig_state = DVAP_INIT;
				else if (rig_type == DVMEGA) rig_state = DVMEGA_INIT;
				else if (rig_type == NODE) rig_state = NODE_INIT;
				rig_error_cnt = 0;
				rig_buff_pnt = 0;
			}
			else
				sig_term = TRUE;
			break;

		case ICOM_INIT:
			memset (inet_frame_id, 0xff, 2);
			icom_init();
			rig_state = ICOM_INIT_DONE;
			time (&rig_init_time);
			rig_alive_send_time = 0;
			rig_send_sw = FALSE;
			while (getFifo(dummy)) ;
			jitter_buff_init();
			break;
	
		case ICOM_INIT_DONE:
			if (icom_init_recv())
			{
				rig_state = HOLE_PUNCH_WAIT;
				syslog (LOG_INFO, "RIG(ID-xxPlus) init/re-init done");
				memset (inet_frame_id, 0x00, 2);
				frameID = 0;
				rig_send_sw = FALSE;
				rig_buff_pnt = 0;
        			rig_last_frame_send = TRUE;
				gettimeofday (&inet_recv_time, NULL);
				Fifo_cnt = 0;
			}
			else
			{
				time (&cur_time);
				if ((cur_time - rig_init_time) >= 2)
				{
        				if(!scan_sw) syslog (LOG_INFO, "RIG(ID-xxPlus) no resp.");
        				else fprintf (status_fd, "RIG(ID-xxPlus) no resp.\n");
					rig_close();
					rig_not_connect_msg();
					not_clear_error_msg = TRUE;
					sig_term = TRUE;
					send_msg_sw = FALSE;
				}
			}
			break;

		case DVAP_INIT:
			dvap_init();
			dvap_check_wait = FALSE;
			rig_state = DVAP_INIT_DONE;
                        length = 1;
                        while (length) length = getFifo(dummy);
                        jitter_buff_init();
			time (&rig_init_time);
			break;
		
		case DVAP_INIT_DONE:
	                if (dvap_check_wait)
			{
				memset (inet_frame_id, 0x00, 2);
				frameID = 0;
				rig_state = HOLE_PUNCH_WAIT;
			}
			else
			{
				time(&cur_time);
				if ((cur_time - rig_init_time) >= 2)
				{
					rig_state = DVAP_INIT;
					rig_error_cnt++;
				}
				if (rig_error_cnt > 3)
				{
                                        if(!scan_sw) syslog (LOG_INFO, "RIG(DVAP) no resp.");
                                        else fprintf (status_fd, "RIG(DVAP) no resp.\n");
                                        rig_close();
					sig_term = TRUE;
					rig_not_connect_msg();
					not_clear_error_msg = TRUE;
					send_msg_sw = FALSE;
				}
			}
			break;

		case DVMEGA_INIT:
			memset (inet_frame_id, 0xff, 2);
                        if (dvmega_init() == 3)
			{
                        	rig_state = DVMEGA_INIT_DONE;
                        	time (&rig_init_time);
                        	rig_alive_send_time = 0;
                        	rig_send_sw = FALSE;
                        	while (getFifo(dummy)) ;
                        	jitter_buff_init();
				rig_buff_pnt = 0;
			}
			else
			{
				rig_error_cnt++;
				if (rig_error_cnt > 3)
                                {
                                        if(!scan_sw) syslog (LOG_INFO, "RIG(DVMEGA) no resp.");
                                        else fprintf (status_fd, "RIG(DVMEGA) no resp.\n");
                                        rig_close();
                                        sig_term = TRUE;
                                        rig_not_connect_msg();
                                        not_clear_error_msg = TRUE;
					send_msg_sw = FALSE;
				}
			}
			break;

		case DVMEGA_INIT_DONE:
                        if (dvmega_init_recv())
                        {
                                rig_state = HOLE_PUNCH_WAIT;
                                syslog (LOG_INFO, "RIG(DVMEGA) init/re-init done");
                                memset (inet_frame_id, 0x00, 2);
                                frameID = 0;
                                rig_send_sw = FALSE;
                                rig_buff_pnt = 0;
                                rig_last_frame_send = TRUE;
                                gettimeofday (&inet_recv_time, NULL);
                                Fifo_cnt = 0;
                        }
                        else
                        {
				time (&cur_time);
                                if ((cur_time - rig_init_time) >= 2)
                                {
                                        if(!scan_sw) syslog (LOG_INFO, "RIG(DVMEGA) no resp.");
                                        else fprintf (status_fd, "RIG(DVMEGA) no resp.\n");
                                        rig_close();
                                        rig_not_connect_msg();
                                        not_clear_error_msg = TRUE;
                                        sig_term = TRUE;
                                        rig_state = HOLE_PUNCH_WAIT;
                                }
                        }
			break;

		case NODE_INIT:
			rig_state = NODE_INIT_DONE;
			break;

		case NODE_INIT_DONE:
			gettimeofday (&inet_recv_time, NULL);
			rig_state = HOLE_PUNCH_WAIT;;
			break;

		case HOLE_PUNCH_WAIT:
			if (hole_punchd_recvd) 
			{
				rig_state = RIG_READ;
			}
			break;

		case RIG_READ:
			if (rig_type == ICOM)
			{
				if (!rig_read())
				{
					rig_state = RIG_OPEN;
					break;
				}
				time (&cur_time);
               		 	if ((cur_time - rig_alive_send_time) >= 1)
                		{
					rig_write (3, alive);
                       			rig_alive_send_time = cur_time;
					digitalWrite (RIG_LED, 0);
               			}
                		if ((cur_time - rig_alive_recv) >= 3)
                		{
                        		icom_reinit();
                		}
			}
			else if (rig_type == DVMEGA) dvmega();
			else if (rig_type == DVAP) dvap();
			else if (rig_type == NODE) node();
			break;
	}
}

int	rig_status_check (void)
{
	if (rig_state == RIG_READ) return TRUE;
	return FALSE;
}

void	icom_reinit(void)
{
	if (error_msg_sw)
	{
		error_msg_sw = TRUE;
		if (!scan_sw) syslog (LOG_INFO, "rig not terminal/access mode");
		else fprintf (status_fd, "rig not terminal/access mode\n");
	}	
	rig_state = ICOM_INIT;
	rig_buff_pnt = 0;
	digitalWrite (RIG_LED, 0);
	digitalWrite (INET_LED, 0);
}

int 	rig_read (void)
{
	int	length;
	int	len;
	int	k;
	int	ret;
	struct	timeval	cu_time;
	struct	timeval	t_time;
	struct	timeval	ack_timeout;
	unsigned short int crc;

	if (FD_ISSET (rig_fd, &read_set))
	{
		length = read (rig_fd, &rig_buff[rig_buff_pnt], 
				sizeof(rig_buff) - rig_buff_pnt);
		if (length < 0)
		{
			syslog (LOG_ERR, "Rig read error %s", strerror(errno));
			if (errno == EAGAIN) return TRUE;
			rig_close();
			return FALSE;
		}
		//time (&rig_alive_send_time);
		rig_buff_pnt += length;
		time (&rig_alive_recv);
		#ifdef	_DEBUG_RIG
		syslog (LOG_INFO, "read length:%d (%d) %2.2x", length, rig_buff_pnt, rig_buff[0]);
		#endif
	}
	while (rig_buff_pnt)
	{
		if (rig_buff[0] != 0xff)
		{
			len = rig_buff[0];
			if (len > rig_buff_pnt) break;

			if (len == 3)
			{
				if (rig_buff[1] == 0x21)
				{
					#ifdef  _DEBUG_RIG
					syslog (LOG_INFO, "ack header:%2.2x seq:%2.2x", rig_buff[1], rig_buff[2]);
					#endif
				}
				else if (rig_buff[1] = 0x03)
				{
					#ifdef	_DEBUG_RIG
					syslog (LOG_INFO, "ack keep alive:%2.2x seq:%2.2x", rig_buff[1], rig_buff[2]);
					#endif
					ptt = rig_buff[2];
				}
				
				k = rig_buff_pnt - len;
				if (k > 0) memmove(&rig_buff[0], &rig_buff[len], k);
				rig_buff_pnt = k;
				break;
			}
			else if (len == 4)
			{
				if (rig_buff[1] == 0x23)
				{
					#ifdef _DEBUG_RIG
					syslog (LOG_INFO, "ack voice:%2.2x seq:%2.2x(%2.2x) %2.2x",
						rig_buff[1], rig_buff[2], seq & 0xff, rig_buff[3]);
					#endif
					if (rig_buff[2] == seq)
					{
						//if (rig_buff[3] == 0x00)
						//{
						//	seq++;
						//	seq &= 0xff;
						//}
                                		k = rig_buff_pnt - len;
                                		if (k > 0) memmove(&rig_buff[0], &rig_buff[len], k);
                                		rig_buff_pnt = k;
                                		break;
					}
					else
					{
						//seq = rig_buff[2] + 1;
						//seq &= 0xff;
					}
					#ifdef	_DEBUG_RIG
					syslog(LOG_INFO, "ack seq error %2.2x %2.2x %2.2x", rig_buff[2], seq, rig_buff[3]);
					#endif
				}
			}
			else if (len == 44)	/* header */
			{
				if (rig_buff[21] == 0x20) return FALSE;
				if (!scan_sw) syslog (LOG_NOTICE, "%8.8s from Rig", &rig_buff[29]);
				else 
				{
					fprintf (status_fd, "%8.8s from Rig\n", &rig_buff[29]);
				}
				//if (!cmd_check (&rig_buff[21])) return FALSE;
				no_cmd_sw = no_cmd_check (&rig_buff[21]);
				if (!rig_status_check()) return FALSE;
				gw_on = FALSE;
				if (!inet_send_buff_set()) return TRUE;
				inet_send_buff[16] = 0x80;
				memcpy (&inet_send_buff[17], &rig_buff[2], 41);
				memcpy (&inet_send_buff[20], area_call, 8); 
				memcpy (&inet_send_buff[28], area_call, 8);
				if (memcmp (&inet_send_buff[36], "CQCQCQ  ", 8)) gw_on = TRUE;
				if (send_sw && !rig_send_sw && no_cmd_sw) 
				{
					send_header_sw = TRUE;
					memset (inet_frame_id, 0xff, 2);
					in_addr.sin_port = htons(dest_inet_port);
					in_addr.sin_addr.s_addr = inet_addr(dest_address);
					m_seq++;
					m_seq &= 0xffff;
        				inet_send_buff[4] = (m_seq >> 8) & 0xff;
        				inet_send_buff[5] = m_seq & 0xff;
					inet_send_buff[7] |= FWD | ZR;
                        		crc = crc_calc (&inet_send_buff[17], 39);
                        		inet_send_buff[56] = (crc >> 8) & 0xff;
                        		inet_send_buff[57] = crc & 0xff;
					ret = sendto (in_addr_sock, inet_send_buff, 58, MSG_DONTWAIT,
						(struct sockaddr *)&in_addr, sizeof(in_addr));
					m_seq++;
					m_seq &= 0xffff;
					inet_send_buff[4] = (m_seq >> 8) & 0xff;
					inet_send_buff[5] = m_seq & 0xff;
					//memcpy (&inet_send_buff[28], area_call, 8);
					//memcpy (&inet_send_buff[20], area_call, 8);
					sendto (in_addr_sock, inet_send_buff, 58, MSG_DONTWAIT,
						(struct sockaddr *)&in_addr, sizeof(in_addr));
					memset (short_msg, 0x20, 20);
					memcpy (used_call, &inet_send_buff[44], 8);
					memcpy (used_call2, &inet_send_buff[52], 4);
					short_msg_wrt();
					if (gw_on)
					{
						memcpy (&inet_send_buff[20], zone_call, 8);
						inet_send_buff[27] = 'G';
						inet_send_buff[7]  &= 0x33;
						inet_send_buff[7] |= GW;
						m_seq++;
						m_seq &= 0xffff;
						inet_send_buff[4] = (m_seq >> 8) & 0xff;
						inet_send_buff[5] = m_seq & 0xff;
                                        	crc = crc_calc (&inet_send_buff[17], 39);
                                        	inet_send_buff[56] = (crc >> 8) & 0xff;
                                        	inet_send_buff[57] = crc & 0xff;
						sendto (in_addr_sock, inet_send_buff, 58, MSG_DONTWAIT,
							(struct sockaddr *)&in_addr, sizeof(in_addr));
						m_seq++;
						m_seq &= 0xffff;
						inet_send_buff[4] = (m_seq >> 8) & 0xff;
						inet_send_buff[5] = m_seq & 0xff;
						sendto (in_addr_sock, inet_send_buff, 58, MSG_DONTWAIT,
							(struct sockaddr *)&in_addr, sizeof(in_addr));
						if ((m_seq % 5) == 0) inet_led_onoff();
					}
				}
				m_seq++;
				m_seq &= 0xffff;
			}
			else if (len == 16)	/* voice */
			{
				inet_send_buff[9] = 0x13;
				inet_send_buff[4] = (m_seq >> 8) & 0xff;
				inet_send_buff[5] = m_seq & 0xff;
				inet_send_buff[7] &= 0x33;
				inet_send_buff[7] |= FWD | ZR;
				if (gw_on) 
				{
					inet_send_buff[7] |= GW;
					if ((m_seq % 5) == 0) inet_led_onoff();
				}
				if (rig_buff[3] & 0x40) memset (&rig_buff[13], 0x55, 3);
				memcpy (&inet_send_buff[16], &rig_buff[3], 13);
				if (send_sw && send_header_sw && no_cmd_sw)
				{
                                        in_addr.sin_port = htons(dest_inet_port);
                                        in_addr.sin_addr.s_addr = inet_addr(dest_address);
					sendto (in_addr_sock, inet_send_buff, 29, MSG_DONTWAIT,
						(struct sockaddr *)&in_addr, sizeof(in_addr));
					dstar_data (inet_send_buff, FALSE);
				}
				else
					dstar_alt_header (inet_send_buff, FALSE);
				if (rig_buff[3] & 0x40) 
				{
					memset (inet_frame_id, 0x00, 2);
					frameID = 0;
					send_header_sw = FALSE;
					digitalWrite (RIG_LED, 0);
					digitalWrite (INET_LED, 0);
				}
				m_seq++;
				m_seq &= 0xffff;
			}
			#ifdef	_DEBUG_RIG
			else 
			{
				syslog (LOG_INFO, "ack length:%d", len);
			}
			#endif

			if (rig_buff[3] == 0x14) rig_led_onoff();
			if ((m_seq % 5) == 0) inet_led_onoff();
		}
		else
			len = 1;
		k = rig_buff_pnt - len;
		if (k > 0) memmove (&rig_buff[0], &rig_buff[len], k);
		rig_buff_pnt = k;
	}
	error_msg_sw = FALSE;

#if 0
	if ((Fifo_cnt > fifo_hold_limit) || (Fifo_cnt < 0))
	{
		syslog (LOG_INFO, "rig fifo overflow (%d)", Fifo_cnt);
		memset (inet_frame_id, 0xff, 2);
		rig_state = RIG_INIT;
		drop_rate_print();
	}
#endif
	return TRUE;
}

void	rig_write (int length, unsigned char buff[])
{
	int ret;
	unsigned char	temp[8];

	if (length == 17)
	{
		seq++;
		seq &= 0xff;
		recv_frame_seq = buff[3];
	}

	if ((length == 42) && (buff[12] =='G'))
	{
		memcpy (temp, &buff[5], 8);
		memcpy (&buff[5], &buff[13], 8);
		memcpy (&buff[13], temp, 8);
	}
	if (rig_type == ICOM)
	{
		if (length == 42) syslog (LOG_DEBUG, "rpt2:%8.8s rpt1:%8.8s ur:%8.8s my:%8.8s my2:%4.4s", &buff[5], &buff[13],&buff[21], &buff[29], &buff[37]);
		ret = write (rig_fd, buff, length);
		if (ret == -1) syslog(LOG_ERR, "rig write error %s", strerror(errno));
		time (&rig_alive_send_time);
	}
	else if (rig_type == DVAP)
	{
		dvap_write (rig_fd, buff, length);
	}
	else if (rig_type == DVMEGA)
	{
		dvmega_write (rig_fd, buff, length);
	}
	else if (rig_type == NODE)
	{
		node_write (buff, length);
	}	
		
	#ifdef	_DEBUG_RIG
	syslog (LOG_INFO, "rig_write %d %d %2.2x %2.2x", length, ret, buff[2], buff[3]);
	#endif
	#ifdef	_DEBUG_RIG_WRITE_PACKET
	if (length == 17)
	{
		syslog (LOG_DEBUG, 
			"%2.2x %2.2x %2.2x  %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %2.2x %d %d",
			buff[0], buff[1], buff[2], buff[3], buff[4], buff[5], buff[6], buff[7], buff[8], buff[9], buff[10], 
			buff[11], buff[12], buff[13], buff[14], buff[15], buff[16], fifo_hold_limit, Fifo_cnt);
	}
	#endif
}

void	icom_init (void)
{
	int	len;

	rig_write (3, init_pkt);
	time (&rig_init_time);
}

void	dvap_reopen (void)
{
	dvap_close ();
	rig_state = RIG_OPEN;
}

void	rig_led_onoff(void)
{
	if (rig_led_sw) rig_led_sw = 0;
	else		rig_led_sw = 1;
	digitalWrite (RIG_LED, rig_led_sw);
}
